function [bestlambda, MKLerror, MKLfscore, MKLnodes] = solveMKL(XTrain, YTrain, XTest, YTest, compfolds, folds, lambdas, dirPath)

global KTYPE
global KSCALE
addpath('~/libsvm-mat');

kfold = size(compfolds,2);
errorMatrix = zeros(length(lambdas),1);
fscoreMatrix = zeros(length(lambdas),1);
p = size(XTrain,2);
myEps = 1e-6;
myTol = 1e-10;
myRidge = 1e-8;

%%%CREATE KERNEL
KTYPE = 1;
KTrain = zeros(length(YTrain),length(YTrain),p);
KTest = zeros(length(YTest),length(YTrain),p);
for i=1:p
    KTrain(:,:,i) = kernel(XTrain(:,i),XTrain(:,i)) +eye(length(YTrain))*myRidge;
    KTest(:,:,i) = kernel(XTest(:,i),XTrain(:,i));
end

%%%CREATE KERNEL

for ifold = 1:kfold
    Yloc = YTrain(compfolds{ifold});
    Ytestloc = YTrain(folds{ifold});
    
    Xloc = XTrain(compfolds{ifold},:);
    Xtestloc = XTrain(folds{ifold},:);
    
    for ilambda = 1:length(lambdas)
        C = 1/lambdas(ilambda);
        w1 = sum(Yloc==1);
        w2 = sum(Yloc== -1);
        n = length(Yloc);
        svm_opt = sprintf('-q -t 4 -c %f -w1 %f -w-1 %f -e %f', C, w2/n, w1/n, myEps);
        %%%MKL - mirror descent
        dualObj = 1.7977e+300;
        ITER_MAX = 500;
        eta = ones(p,1)/p;
        for iter_gamma=1:ITER_MAX
            dualObjOld = dualObj;
            etaOld = eta;
            K1 = zeros(n,n);
            for i=1:p
                K1 = K1 + KTrain(compfolds{ifold},compfolds{ifold},i)*eta(i);
            end
            fprintf('.');
            model = svmtrain(Yloc, [(1:n)',K1], svm_opt);
            fprintf('*');
            alphaOpt = zeros(n,1);
            alphaOpt(model.SVs) = abs(model.sv_coef);
            alphaOpt = alphaOpt.*Yloc;
            bOpt = -model.rho;
            if (model.Label(1) == -1)
                bOpt = -bOpt;
            end
            dualObj = (sum(abs(alphaOpt)) - .5*alphaOpt'*K1*alphaOpt);
            if (dualObj < 0)
                fprintf('Dual Obj is negative in MKL\n');
                keyboard;
            end
            aTKa = zeros(p,1);
            for i=1:p
                aTKa(i) = alphaOpt'*KTrain(compfolds{ifold},compfolds{ifold},i)*alphaOpt;
            end
            gradF = -.5*aTKa;
            step = sqrt(log(p))/(sqrt(iter_gamma)*norm(gradF,inf));
            pvec = 1+log(eta) - step*gradF;
            eta = exp(pvec);
            eta = eta/sum(eta);
%             eta(find(eta<myTol/length(eta))) = 0;
%             eta = eta/sum(eta);
            if (dualObjOld <= dualObj)
                fprintf('This is bad - in MKL! dualObjOld=%f, dualObj=%f\n',dualObjOld, dualObj);
            end
            if (abs((dualObjOld-dualObj)/dualObj) < myEps)
                break;
            end
        end
        fprintf('\n');
        if (iter_gamma == ITER_MAX)
            fprintf('Before convergence termination of MD in MKL, diff = %f\n',abs((dualObjOld-dualObj)/dualObj));
        end
        inactive = eta <= myTol;
        eta(inactive) = 0;

        predtest = zeros(length(Ytestloc),1)+bOpt;
        
        for i=1:p
            try
                predtest = predtest + eta(i)*(KTrain(folds{ifold},compfolds{ifold},i)*alphaOpt);
            catch exception
                keyboard;
            end
        end
        predY = sign(predtest);
        errorMatrix(ilambda) = errorMatrix(ilambda,1) + (sum(abs(predY-Ytestloc))/2)/length(Ytestloc);
        fscoreMatrix(ilambda) = fscoreMatrix(ilambda,1) + calculateFscore(predtest,Ytestloc);
    end
end

% [m index] = min(errorMatrix);
% bestlambda = lambdas(index);
[m index] = max(fscoreMatrix);
bestlambda = lambdas(index);

% keyboard;

bestC = 1/bestlambda;
w1 = sum(YTrain==1);
w2 = sum(YTrain== -1);
n = length(YTrain);
svm_opt = sprintf('-q -t 4 -c %f -w1 %f -w-1 %f -e %f', bestC, w2/n, w1/n, myEps);
%%%MKL - mirror descent    
dualObj = 1.7977e+300;    
ITER_MAX = 200;
eta = ones(p,1)/p;
for iter_gamma=1:ITER_MAX
    dualObjOld = dualObj;
    etaOld = eta;
    K1 = zeros(n,n);
    for i=1:p
        K1 = K1 + KTrain(:,:,i)*eta(i);
    end
    fprintf('.');
    model = svmtrain(YTrain, [(1:n)',K1], svm_opt);
    fprintf('*');
    alphaOpt = zeros(n,1);
    alphaOpt(model.SVs) = abs(model.sv_coef);
    alphaOpt = alphaOpt.*YTrain;
    bOpt = -model.rho;
    if (model.Label(1) == -1)
        bOpt = -bOpt;
    end
    dualObj = (sum(abs(alphaOpt)) - .5*alphaOpt'*K1*alphaOpt);
    if (dualObj < 0)
        fprintf('Dual Obj is negative in MKL\n');
        keyboard;
    end
    aTKa = zeros(p,1);
    for i=1:p
        aTKa(i) = alphaOpt'*KTrain(:,:,i)*alphaOpt;
    end
    gradF = -.5*aTKa;
    step = sqrt(log(p))/(sqrt(iter_gamma)*norm(gradF,inf));
    pvec = 1+log(eta) - step*gradF;
    eta = exp(pvec);
    eta = eta/sum(eta);
%     eta(find(eta<myTol/length(eta))) = 0;
%     eta = eta/sum(eta);
    if (dualObjOld <= dualObj)
        fprintf('This is bad - in MKL! dualObjOld=%f, dualObj=%f\n',dualObjOld, dualObj);
    end
    if (abs((dualObjOld-dualObj)/dualObj) < myEps)
        break;
    end
end
fprintf('\n');
if (iter_gamma == ITER_MAX)
    fprintf('Before convergence termination of MD in MKL, diff = %f\n',abs((dualObjOld-dualObj)/dualObj));
end
inactive = eta <= myTol;
eta(inactive) = 0;

predtest = zeros(length(YTest),1)+bOpt;
MKLnodes = sum(eta > myTol);
for i=1:p
    predtest = predtest + eta(i)*(KTest(:,:,i)*alphaOpt);
end

predY = sign(predtest);
MKLerror = (sum(abs(predY-YTest))/2)/length(YTest);
MKLfscore = calculateFscore(predtest,YTest);
% keyboard;

if ~isempty(dir)
    filename = strcat(dirPath, 'MKLResult.mat');
    save(filename, 'kfold', 'lambdas', 'svm_opt', 'MKLerror', 'eta', 'bestlambda', 'errorMatrix', 'fscoreMatrix', 'MKLfscore', 'MKLnodes', 'predtest');
end

fprintf('\nMKLerror: %f\n', MKLerror);
fprintf('\nMKLfscore: %f\n', MKLfscore);


% pos = find(YTrain==1);
% neg = find(YTrain== -1);

% cvx_begin
%     variable alpha1(n);
%     variable t;
%     %     minimize 0.5*square(t1) + sum(epsilon1)/lambda
%     maximize sum(alpha1) - .5*t
%     subject to
%         for i=1:p
%             alpha1'*diag(YTrain)*KTrain(:,:,i)*diag(YTrain)*alpha1 <= t;
%         end
%         alpha1'*YTrain >= 0;
%         alpha1'*YTrain <= 0;
%         0 <= alpha1;
%         alpha1(pos) <= bestC*w2/n;
%         alpha1(neg) <= bestC*w1/n;
% cvx_end

% cvx_begin
%     variable w(p);
%     variable b;
%     variable epsilon1(n);
%     variable t;
%     minimize .5*square(t) + sum(epsilon1(pos))*bestC*w2/n +  sum(epsilon1(neg))*bestC*w1/n
%     subject to
%         epsilon1(pos) >= 1-YTrain(pos).*((XTrain(pos,:)*w+b));
%         epsilon1(neg) >= 1-YTrain(neg).*((XTrain(neg,:)*w+b));
%         epsilon1 >= 0;
%         norm(w,1)<=t;
% cvx_end

% fprintf('\nDual Obj: %f\n', dualObj);

% keyboard;
