function [bestlambda, SVMerror, SVMfscore]=solveSVM(XTrain, YTrain, XTest, YTest, compfolds, folds, lambdas, dirPath)

addpath('~/libsvm-mat');
gammas = logspace(-3,3,7);
myEps = 1e-6;
% CV for gamma, CV for C

kfold = size(compfolds,2);
errorMatrix = zeros(length(lambdas),length(gammas));
fscoreMatrix = zeros(length(lambdas),length(gammas));
for ifold = 1:kfold
    Yloc = YTrain(compfolds{ifold});
    Ytestloc = YTrain(folds{ifold});

    Xloc = XTrain(compfolds{ifold},:);
    Xtestloc = XTrain(folds{ifold},:);
    
    for ilambda = 1:length(lambdas)
        C = 1/lambdas(ilambda);
        for igamma = 1:length(gammas)
            omega = gammas(igamma);
            w1 = sum(Yloc==1);
            w2 = sum(Yloc== -1);
            n = length(Yloc);
            svm_opt = sprintf('-q -s 0 -t 2 -g %f -c %f -w1 %f -w-1 %f -e %f', omega, C, w2/n, w1/n, myEps);% WEIGHTING
            model = svmtrain(Yloc, Xloc, svm_opt);
            [predictedY, accuracy, decisionVal] = svmpredict(Ytestloc, Xtestloc, model);
            errorMatrix(ilambda,igamma) = errorMatrix(ilambda,igamma) + 1-accuracy(1)/100;
            fscoreMatrix(ilambda,igamma) = fscoreMatrix(ilambda,igamma) + calculateFscore(decisionVal,Ytestloc);
        end
    end
end

% [m rowVal] = min(errorMatrix,[],1);
% [minErr col] = min(min(errorMatrix));
[m rowVal] = max(fscoreMatrix,[],1);
[maxFs col] = max(max(fscoreMatrix));

row = rowVal(col);
bestlambda = lambdas(row);
bestC =  1/bestlambda;
bestOmega = gammas(col);
w1 = sum(YTrain==1);
w2 = sum(YTrain== -1);
n = length(YTrain);
svm_opt = sprintf('-q -s 0 -t 2 -g %f -c %f -w1 %f -w-1 %f -e %f', bestOmega, bestC, w2/n, w1/n, myEps);% WEIGHTING
model = svmtrain(YTrain, XTrain, svm_opt);
[svmPredY, svmAccuracy, decisionVal] = svmpredict(YTest, XTest, model);
SVMerror = 1-svmAccuracy(1)/100;
SVMfscore =  calculateFscore(decisionVal,YTest);
filename = strcat(dirPath, 'SVMResult.mat');
save(filename, 'kfold', 'gammas', 'lambdas', 'svm_opt', 'SVMerror', 'model', 'bestlambda', 'bestOmega', 'errorMatrix', 'SVMfscore', 'fscoreMatrix',...
    'svmPredY', 'decisionVal');
fprintf('\nSVMerror: %f\n', SVMerror);
fprintf('\nSVMfscore: %f\n', SVMfscore);
