function [alphaOpt,bOpt,eta,omegaSquared,finalLpDualObj,theta,zeta,aTKa] = solve_ghkl_hinge_shogun(eta,data,Y,lambda,rho,varargin)

% solve the milasso for a given regularization parameter
n = size(data.X,1);
p =length(data.affinity);
affs = data.affinity;
weights = data.weights;
fprintf('\nNo. of nodes: %d\n',p);
X = data.X;	% no need to center here for logistic
w1 = sum(Y == 1);
w2 = sum(Y == -1);
if (w1+w2 ~= n)
    'Problem IN SMALL PROBLEM SOLVER'
    keyboard;
end
%Calculate Kernel
myRidge = 1e-8;
myEps = 1e-6;%1e-12;

K = zeros(n,n,p);
for i=1:p
    K(:,:,i) = X(:,i)*X(:,i)';
    if (~any(K(:,:,i)))
        fprint('\nALL ZERO MATRIX FOR %d th feature\n',i);
    end
    K(:,:,i) = K(:,:,i) +eye(n)*myRidge;
end

tic;
if (p > 1)
    if (rho ~= 2)
        rhobar = rho/(2-rho);
        rhobarstar = rhobar/(rhobar-1);
        mkl_norm = rho/(2-rho);
    else
        rhobar = inf;
        rhobarstar = 1;
        mkl_norm = inf;
    end
    ITER_MAX = 500;

%     % WARM START
    eta = eta+min(.05,max(eta/2));
    eta = eta/sum(eta);
    %COLD START
%     eta = ones(p,1)/p;
    % MIRROR DESCENT
    dualObj = 1.7977e+300;
    min_alpha = nan;
    min_b = nan;
    min_eta = nan;
    min_zeta = nan;
    min_dualObj = 1.7977e+300;
    min_delta = nan;
    for iter_gamma = 1:ITER_MAX
        % Solve for alpha ;given eta
        dualObjOld = dualObj;
        etaOld = eta;
        if (sum(isnan(eta)))
            'ETA NAN'
            keyboard;
        end
        fprintf('.');
        zetaFake = calculateZetaFake(affs,rho,weights,eta);
        zeta = nthroot(zetaFake,rhobarstar);
        nonZero = find(zetaFake>0);
        K1 = zeros(n,n,p);
        for i=1:p
            K1(:,:,i) = K(:,:,i)*zeta(i);
        end
        if (rho ~= 2)
            % USE SHOGUN
            try
                fprintf('*');
                [svmProp, bOpt, shoDual, shoPrimal] = lpmkl(K1(:,:,nonZero),ones(length(nonZero),1),1/lambda,mkl_norm,Y,w1/n,w2/n);% -ve, +ve
                fprintf('*');
            catch exception
                'EXCEPTION SHOGUN'
                keyboard;
            end
            svIndex = round(svmProp(:,2)+1);
            alphaOpt = zeros(n,1);
            alphaOpt(svIndex,1) = svmProp(:,1);
            aTKa1 = zeros(p,1);
            for i=1:p
                aTKa1(i) = alphaOpt'*K1(:,:,i)*alphaOpt;
            end
            dualObj = sum(abs(alphaOpt)) - .5*norm(aTKa1,rhobarstar);
            if (dualObj < 0)
                fprintf('Dual Obj is negative\n');
                keyboard;
            end
        else
            % USE LIBVSM - SOLVE L2-SVM DUAL
            K2 = zeros(n,n);
            for i=1:p
                K2 = K2 + K1(:,:,i);
            end
            fprintf('*');
            svm_opt = sprintf('-q -t 4 -c %f -w1 %f -w-1 %f -e %f', 1/lambda, w2/n, w1/n, myEps);
            model = svmtrain(Y, [(1:n)',K2], svm_opt);
            fprintf('*');
            alphaOpt = zeros(n,1);
            alphaOpt(model.SVs) = abs(model.sv_coef);
            alphaOpt = alphaOpt.*Y;
            bOpt = -model.rho;
            if (model.Label(1) == -1)
                bOpt = -bOpt;
            end
            dualObj = (sum(abs(alphaOpt)) - .5*alphaOpt'*K2*alphaOpt);
            if (dualObj < 0)
                fprintf('Dual Obj is negative\n');
                keyboard;
            end
        end
        
        delta = 2*(sum(abs(alphaOpt)) - dualObj);
        if (delta <= 0)
            fprintf('Delta is negative\n');
            keyboard;
        end
        if ( 1e-6 <= (dualObj-dualObjOld)/dualObj)
            fprintf('This is bad! dualObjOld=%e, dualObj=%e, rel.gap=%e\n',dualObjOld, dualObj,(dualObj-dualObjOld)/dualObj);
%             keyboard;
        end
        %book-keeping
        if (dualObj < min_dualObj)
            min_alpha = alphaOpt;
            min_b = bOpt;
            min_eta = eta;
            min_zeta = zeta;
            min_dualObj = dualObj;
            min_delta = delta;
        end
        %book-keeping
        if (abs((dualObjOld-dualObj)/dualObj) < myEps)
            break;
        end
        T2 = delta^rhobarstar;
        gradF = -(delta/T2)/(2*rhobarstar)*calculateSumForDesc(alphaOpt,affs,rho,weights,eta,K,zetaFake);
        step = sqrt(log(p))/(sqrt(iter_gamma)*norm(gradF,inf));
        pvec = 1+log(eta) - step*gradF;
        eta = exp(pvec);
        eta = eta/sum(eta);
    end
    if (iter_gamma == ITER_MAX)
        fprintf('Before convergence termination of MD, diff = %e\n',abs((dualObjOld-dualObj)/dualObj));
    end
    alphaOpt = min_alpha;
    bOpt = min_b;
    eta  = min_eta;
    zeta = min_zeta;
    dualObj = min_dualObj;
    delta = min_delta;
    
    finalLpDualObj = dualObj;
    fprintf('\nCOLD START ITERATION: %d, OBJECTIVE: %e, ACTIVE: %d\n', iter_gamma, dualObj, sum(eta>1e-12));


    % Theta Computation
    theta = zeros(p,1);
    aTKa = zeros(p,1);
    for i=1:p
        aTKa(i) = alphaOpt'*K1(:,:,i)*alphaOpt;
        try
            theta(i) = nthroot(aTKa(i)/delta,mkl_norm-1);
        catch exception
            'THETA CALCULATION'
            keyboard;
        end
    end
    omegaSquared = delta;
end

% for p==1 use CVX
if (p == 1)
    [alphaOpt,bOpt,eta,omegaSquared,finalLpDualObj,theta,zeta,aTKa] = cvxLpNormSolver(eta,data,Y,lambda,rho,K);
end

fprintf('Time in solving problem: ');
toc;

omegaSquared = omegaSquared*lambda/2;
finalLpDualObj = finalLpDualObj*lambda;
fprintf('\nObjective = %e\n',finalLpDualObj);

%%%END function solve_ghkl_hinge_shogun

function [zetaFake] = calculateZetaFake(affs,rho,weights,eta)
p =length(eta);
mat = zeros(p,p);
for i=1:p
    mat(i,affs{i}) = 1;
end
dffs = cell(p,1);
for i=1:p
    dffs{i} = find(mat(:,i)==1);
end

weightRho = weights.^(rho/(1-rho));
zetaFake = zeros(p,1);
etaFake = weightRho.*eta;
for i=1:p
    zetaFake(i,1) = norm(etaFake(dffs{i}),1-rho);
end


function [answer] = calculateSumForDesc(alphaOpt,affs,rho,weights,eta,K,zetaFake)

p =length(eta);
etaRho = eta.^rho;
weightRho = weights.^rho;

if (rho ~= 2)
    rhobar = rho/(2-rho);
    rhobarstar = rhobar/(rhobar-1);
else
    rhobar = inf;
    rhobarstar = 1;
end

aTKa = zeros(p,1);
for i=1:p
    aTKa(i) = alphaOpt'*K(:,:,i)*alphaOpt;
end
aTKaZeta = (aTKa.^(rhobarstar)).*(zetaFake.^(rho));

answer = zeros(p,1);
for i=1:p
    answer(i) = sum(aTKaZeta(affs{i}));
end

for i=1:p
    if (etaRho(i) > 0)
        answer(i) = answer(i)*weightRho(i)/etaRho(i);
    else
        'ETARHO IS ZERO'
        keyboard;
    end
end

